@@ -163,6 +163,11 @@ fill.NAs <- function(x, data = NULL, all.covs = FALSE, contrasts.arg=NULL) {
     }
   }
 
+  # Adding informative error for #104 prior to fixing the overarching issue
+  if (any(sapply(data, function(x) !is.null(dim(x))))) {
+    stop("Data with matrix columns not supported; potentially generated by `cbind` inside formula.")
+  }
+
   withStrata <- findStrata(x, data)
 
   data <- as.data.frame(data) # in case it is a matrix
