@@ -39,11 +39,7 @@
 #' returned by \code{match_on}. This argument can reduce the processing time
 #' needed to compute sparse distance matrices.
 #'
-#' The \code{match_on} function is similar to the older, but still supplied,
-#' \code{\link{mdist}} function. Future development will concentrate on
-#' \code{match_on}, but \code{mdist} is still supplied for users familiar with
-#' the interface. For the most part, the two functions can be used
-#' interchangeably by users.
+#' Details for each particular first type of argument follow:
 #'
 #' @param x An object defining how to create the distances. All methods require
 #'   some form of names (e.g. \code{names} for vectors or \code{rownames} for
@@ -85,15 +81,15 @@ match_on <- function(x, within = NULL, caliper = NULL, exclude = NULL, data=NULL
   UseMethod("match_on")
 }
 
-#' @details The \code{glm} method assumes its first argument to be a fitted
-#'   propensity model. From this it extracts distances on the \emph{linear}
-#'   propensity score: fitted values of the linear predictor, the link function
-#'   applied to the estimated conditional probabilities, as opposed to the
-#'   estimated conditional probabilities themselves (Rosenbaum \& Rubin, 1985).
-#'   For example, a logistic model (\code{glm} with \code{family=binomial()})
-#'   has the logit function as its link, so from such models \code{match_on}
-#'   computes distances in terms of logits of the estimated conditional
-#'   probabilities, i.e. the estimated log odds.
+#' @details \bold{First argument (\code{x}): \code{glm}.} The model is assumed to be
+#'   a fitted propensity score model. From this it extracts distances on the
+#'   \emph{linear} propensity score: fitted values of the linear predictor, the
+#'   link function applied to the estimated conditional probabilities, as opposed
+#'   to the estimated conditional probabilities themselves (Rosenbaum \& Rubin,
+#'   1985).  For example, a logistic model (\code{glm} with
+#'   \code{family=binomial()}) has the logit function as its link, so from such
+#'   models \code{match_on} computes distances in terms of logits of the
+#'   estimated conditional probabilities, i.e. the estimated log odds.
 #'
 #'   Optionally these distances are also rescaled. The default is to rescale, by
 #'   the reciprocal of an outlier-resistant variant of the pooled s.d. of
@@ -179,10 +175,10 @@ match_on_szn_scale <- function(x, Tx, standardizer = mad, ...) {
   }
 }
 
-#' @details The \code{bigglm}
-#' method works analogously to the \code{glm} method, but with \code{bigglm} objects, created by
-#' the \code{bigglm} function from package \sQuote{biglm}, which can
-#' handle bigger data sets than the ordinary glm function can.
+#' @details \bold{First argument (\code{x}): \code{bigglm}.} This method works
+#'   analogously to the \code{glm} method, but with \code{bigglm} objects,
+#'   created by the \code{bigglm} function from package \sQuote{biglm}, which can
+#'   handle bigger data sets than the ordinary glm function can.
 #'
 #' @method match_on bigglm
 #' @rdname match_on-methods
@@ -221,18 +217,19 @@ are there missing values in data?")
   match_on(theps, within = within, caliper = caliper, exclude = exclude, z = z, ... )
 }
 
-#' @details The formula method produces, by default, a Mahalanobis distance
-#'   specification based on the formula \code{Z ~ X1 + X2 + ... }, where
-#'   \code{Z} is the treatment indicator. The Mahalanobis distance is calculated
-#'   as the square root of d'Cd, where d is the vector of X-differences on a
-#'   pair of observations and C is an inverse (generalized inverse) of the
-#'   pooled covariance of Xes. (The pooling is of the covariance of X within the
-#'   subset defined by \code{Z==0} and within the complement of that
-#'   subset. This is similar to a Euclidean distance calculated after
-#'   reexpressing the Xes in standard units, such that the reexpressed variables
-#'   all have pooled SDs of 1; except that it addresses redundancies among the
-#'   variables by scaling down variables contributions in proportion to their
-#'   correlations with other included variables.)
+#' @details \bold{First argument (\code{x}): \code{formula}.} The formula must have
+#'   \code{Z}, the treatment indicator (\code{Z=0} incidates control group,
+#'   \code{Z=1} indicates treatment group), on the left hand side, and any
+#'   variables to compute a distance on on the right hand side. E.g. \code{Z ~ X1
+#'   + X2}. The Mahalanobis distance is calculated as the square root of d'Cd,
+#'   where d is the vector of X-differences on a pair of observations and C is an
+#'   inverse (generalized inverse) of the pooled covariance of Xes. (The pooling
+#'   is of the covariance of X within the subset defined by \code{Z==0} and
+#'   within the complement of that subset. This is similar to a Euclidean
+#'   distance calculated after reexpressing the Xes in standard units, such that
+#'   the reexpressed variables all have pooled SDs of 1; except that it addresses
+#'   redundancies among the variables by scaling down variables contributions in
+#'   proportion to their correlations with other included variables.)
 #'
 #'   Euclidean distance is also available, via \code{method="euclidean"}, and
 #'   ranked, Mahalanobis distance, via \code{method="rank_mahalanobis"}.
@@ -257,8 +254,7 @@ are there missing values in data?")
 #'   specification.
 #' @param method A string indicating which method to use in computing the
 #'   distances from the data.  The current possibilities are
-#'   \code{"mahalanobis", "euclidean", "rank_mahalanobis"}, or pass a user
-#'   created distance function.
+#'   \code{"mahalanobis", "euclidean"} or \code{"rank_mahalanobis"}.
 #' @method match_on formula
 #' @rdname match_on-methods
 #' @export
@@ -359,7 +355,10 @@ match_on.formula <- function(x, within = NULL, caliper = NULL, exclude = NULL, d
 		makedist(z, data, compute_mahalanobis, within),
 		makedist(z, data, compute_euclidean, within),
     makedist(z, data, compute_rank_mahalanobis, within),
-		makedist(z, data, match.fun(method), within)
+    {
+      warning("Passing a user-defined `method` to `match_on.formula` is not supported and results are not guaranteed. User-defined distances should use `match_on.function` instead.")
+      makedist(z, data, match.fun(method), within)
+    }
 		)
   rm(mf)
 
@@ -375,7 +374,7 @@ match_on.formula <- function(x, within = NULL, caliper = NULL, exclude = NULL, d
     {
       tmp@groups <- unlist(list(tmp@groups, within@groups[!names(within@groups) %in% names(tmp@groups)]))
     }
-  
+
   }
 
   if (is.null(caliper)) {
@@ -393,7 +392,7 @@ match_on.formula <- function(x, within = NULL, caliper = NULL, exclude = NULL, d
 # and the data frame, return an updated formula without the strata elements
 # and an exactMatch using the strata.
 #
-# Note: Only use the `x` returned in match_on.formula; in match_on.glm we do
+# Note: Only use the \code{x} returned in match_on.formula; in match_on.glm we do
 # not remove the strata variables from the model.
 makeWithinFromStrata <- function(x, data)
 {
@@ -506,21 +505,20 @@ compute_rank_mahalanobis <- function(index, data, z) {
     return(rankdists)
 }
 
-#' @details The \code{function} method takes as its \code{x} argument a function
-#'   of three arguments: \code{index}, \code{data}, and \code{z}. The
-#'   \code{data} and \code{z} arguments will be the same as those passed
-#'   directly to \code{match_on}. The \code{index} argument is a matrix of two
-#'   columns, representing the pairs of treated and control units that are valid
-#'   comparisons (given any \code{within} arguments). The first column is the
-#'   row name or id of the treated unit in the \code{data} object. The second
-#'   column is the id for the control unit, again in the \code{data} object. For
-#'   each of these pairs, the function should return the distance between the
-#'   treated unit and control unit.  This may sound complicated, but is simple
-#'   to use. For example, a function that returned the absolute difference
-#'   between two units using a vector of data would be \code{
-#'   f <- function(index, data, z) { abs(data[index[,1]] - data[index[,2]]) }
-#'   }.  (Note: This simple case is precisely
-#'   handled by the \code{numeric} method.)
+#' @details \bold{First argument (\code{x}): \code{function}.} The passed function
+#'   must take arguments: \code{index}, \code{data}, and \code{z}. The
+#'   \code{data} and \code{z} arguments will be the same as those passed directly
+#'   to \code{match_on}. The \code{index} argument is a matrix of two columns,
+#'   representing the pairs of treated and control units that are valid
+#'   comparisons (given any \code{within} arguments). The first column is the row
+#'   name or id of the treated unit in the \code{data} object. The second column
+#'   is the id for the control unit, again in the \code{data} object. For each of
+#'   these pairs, the function should return the distance between the treated
+#'   unit and control unit.  This may sound complicated, but is simple to
+#'   use. For example, a function that returned the absolute difference between
+#'   two units using a vector of data would be \code{ f <- function(index, data,
+#'   z) { abs(data[index[,1]] - data[index[,2]]) } }.  (Note: This simple case is
+#'   precisely handled by the \code{numeric} method.)
 #'
 #' @param z A logical or binary vector indicating treatment and control for each
 #'  unit in the study. TRUE or 1 represents a treatment unit, FALSE of 0 represents
@@ -555,13 +553,13 @@ match_on.function <- function(x, within = NULL, caliper = NULL, exclude = NULL,
 
 # Note that Details for the glm method, above, refers to the below for discussion of computational
 # benefits of calipers -- if that's changed here, adjust there accordingly.
-#' @details The \code{numeric} method returns absolute differences between treated and control units'
+#' @details \bold{First argument (\code{x}): \code{numeric}.} This returns absolute differences between treated and control units'
 #' values of \code{x}. If a caliper is specified, pairings with \code{x}-differences greater than it
 #' are forbidden.  Conceptually, those distances are set to \code{Inf}; computationally, if either of
 #' \code{caliper} and \code{within} has been specified then only information about permissible pairings
 #' will be stored, so the forbidden pairings are simply omitted. Providing a \code{caliper} argument here,
 #' as opposed to omitting it and afterward applying the \code{\link{caliper}} function, reduces
-#' storage requirements and may otherwise improve performance, particularly in larger problems. 
+#' storage requirements and may otherwise improve performance, particularly in larger problems.
 #'
 #' For the numeric method, \code{x} must have names.
 #' @method match_on numeric
@@ -709,7 +707,7 @@ scoreCaliper <- function(x, z, caliper) {
   makeInfinitySparseMatrix(rep(0, length(treatedids)), controlids, treatedids, names(control), names(treated))
 }
 
-#' @details The \code{matrix} and \code{InfinitySparseMatrix} just return their
+#' @details \bold{First argument (\code{x}): \code{matrix} or \code{InfinitySparseMatrix}.} These just return their
 #'   arguments as these objects are already valid distance specifications.
 #'
 #' @rdname match_on-methods
