@@ -52,7 +52,8 @@ summary.optmatch <- function(object,
   if (all(mfd))
     {
       class(so) <- "summary.optmatch"
-      so$matching.failed <- table(subprobs, attr(object, "contrast.group"))
+      so$matching.failed <- table(subprobs, attr(object, "contrast.group"),
+                                  exclude = NA, useNA = 'no')
       dimnames(so$matching.failed)[[2]] <- c("z==0", "z==1")
       so$warnings <- c(so$warnings,
                        list("Matching failed.  (Restrictions impossible to meet?)\nEnter ?matchfailed for more info.")
@@ -61,7 +62,7 @@ summary.optmatch <- function(object,
     }
   match.succeed <- tapply(mfd, subprobs, function(x) !all(x))
   so$matching.failed <- table(subprobs, attr(object, "contrast.group"),
-                              exclude = names(match.succeed)[match.succeed],
+                              exclude = c(names(match.succeed)[match.succeed], NA), 
                               useNA = 'no')
   if (prod(dim(so$matching.failed)) == 0) {
     so$matching.failed <- NULL
